# CI-V Simple Split controller and icom memory keyer subset
# 2020/12/15 ver1.0 完成版

# programmed by Toshi JA0RUG

import serial
import tkinter as tk
import tkinter.messagebox as txtmsg
import time

wpm = 25 # CW速度の初期値(6～48)

RIG = 0x00 # ICOM RIGのCI-Vアドレス.0x00に設定してあるが,使用
           # 無線機のCI-Vアドレスに変えることが望ましい.
PC  = 0xe1 # PCのCI-Vアドレス 現在は 0xe1に設定

RIG_COM  = 'COM4' # RIGを接続したCOMポート番号
RIG_BAUD = 115200 # RIGのCI-Vボーレート

# メッセージ内容は編集して使う
msg = [b'JA0RUG', b'5NNTU', b'TU5NN5NNBK', b'message 4']

# global変数[modeの初期値, xfcの初期値, 周波数の初期値(14.195)]
settings = ['Simplex', 'XFC OFF', 14.195]

# COMポートの設定
try:
    comport = serial.Serial('\\\\.\\' + RIG_COM, RIG_BAUD, timeout = None)
except Exception as e:
    txtmsg.showerror('確認', RIG_COM + 'が開けません. 終了します.')
    exit()

# wpm:speed CWスピード設定用辞書
wpm_speed = {6:[0x00,0x00],  7:[0x00,0x07],  8:[0x00,0x12],  9:[0x00,0x19], 10:[0x00,0x25],

            11:[0x00,0x31], 12:[0x00,0x37], 13:[0x00,0x43], 14:[0x00,0x49], 15:[0x00,0x55],

            16:[0x00,0x61], 17:[0x00,0x67], 18:[0x00,0x73], 19:[0x00,0x79], 20:[0x00,0x84],

            21:[0x00,0x91], 22:[0x00,0x97], 23:[0x01,0x03], 24:[0x01,0x08], 25:[0x01,0x14],

            26:[0x01,0x21], 27:[0x01,0x28], 28:[0x01,0x34], 29:[0x01,0x40], 30:[0x01,0x44],

            31:[0x01,0x51], 32:[0x01,0x56], 33:[0x01,0x64], 34:[0x01,0x69], 35:[0x01,0x75],

            36:[0x01,0x82], 37:[0x01,0x88], 38:[0x01,0x92], 39:[0x01,0x99], 40:[0x02,0x03],

            41:[0x02,0x11], 42:[0x02,0x15], 43:[0x02,0x24], 44:[0x02,0x29], 45:[0x02,0x34],

            46:[0x02,0x39], 47:[0x02,0x44], 48:[0x02,0x50]}

def kbd(event): #ファンクションキーなどによる送信 ファンクションキーはF1～F8を使用
    key  = event.keysym

    if key =='Escape': # ESCキーは送信停止キー
        tx_off()
        print('tx off')

    elif key =='F1':
        send_msg(0)
        print('send_msg 1')

    elif key =='F2':
        send_msg(1)
        print('send_msg 2')

    elif key =='F3':
        send_msg(2)
        print('send_msg 3')

    elif key =='F4':
        send_msg(3)
        print('send_msg 4')        

    elif key =='F11': # 初期化(VFO-A選択, VFO-A --> VFO-B, Split ON)
        if settings[0] == 'Simplex':
            set_split()
            settings[1] = 'XFC OFF'
            settings[0] = 'Split'
        else:
            set_simplex()
            settings[0] = 'Simplex'
            settings[1] = ''

        print(settings[0])
        var1.set(settings[0])
        var2.set(settings[1])

    elif key =='F12': # Split解除, プログラム終了
        exit_proc()

    elif key =='Up':   # ↑キーはCW Speed up
        set_wpm(1)

    elif key =='Down': # ↓キーはCW Speed down
        set_wpm(-1)

def set_wpm(spd):
    global wpm

    wpm = wpm + spd

    if wpm < 6:
        wpm =6 

    elif wpm > 48:
        wpm = 48

    comport.write([0xfe, 0xfe, RIG, PC, 0x14, 0x0c, wpm_speed[wpm][0], wpm_speed[wpm][1], 0xfd])

    var0.set(wpm)

def send_msg(msg_nr):
    comport.write([0xfe, 0xfe, RIG, PC, 0x17]) 
    comport.write(msg[msg_nr])
    comport.write([0xfd])

def tx_off():
    comport.write([0xfe, 0xfe, RIG, PC, 0x17, 0xff, 0xfd])

def set_simplex():
    comport.write([0xfe, 0xfe, RIG, PC, 0x0f, 0x00, 0xfd])

def set_split():
    comport.write([0xfe, 0xfe, RIG, PC, 0x07, 0x00, 0xfd])
    time.sleep(0.01)
    comport.write([0xfe, 0xfe, RIG, PC, 0x07, 0xa0, 0xfd])
    time.sleep(0.01)
    comport.write([0xfe, 0xfe, RIG, PC, 0x0f, 0x01, 0xfd]) 
    time.sleep(0.01)
    read_freq()
 
def read_freq():

    buff1 = ''
    buff2 = ''

    comport.write([0xfe, 0xfe, RIG, PC, 0x03, 0xfd])

    time.sleep(0.01)

    while True:
        buff1  = comport.read().hex()
        buff2 = buff2 + buff1

        if buff1 == 'fd' and buff2[8:10] == '03':

            settings[2] = float(buff2[16:18]  + '.' + buff2[14:16] + buff2[12:13]   +  buff2[13:14] + buff2[10:12])
            print(settings[2])   
            break

        elif buff1 == 'fd' and buff2[8:10] != '03':
            buff2 = ''
            continue

def set_freq():
    data = '{:011.06f}'.format(settings[2])
    comport.write([0xfe, 0xfe, RIG, PC, 0x00])
    comport.write([int(data[9:11], 16), int(data[7:9], 16), int(data[5:7], 16), int(data[2:4], 16), int(data[0:2], 16)])
    comport.write([0xfd])

def mouse_wheel(event):

    count = 0

    if event.delta == -120:
        count = -0.0001

    elif event.delta ==  120:
        count =  0.0001

    elif event.delta >=  240:
        count =  0.0002

    elif event.delta == -240:
        count = -0.0002

    elif event.delta <= -360:
        count = -0.0003

    elif event.delta >=  360:
        count =  0.0003

    if settings[1] == 'XFC ON':
        settings[2] = settings[2] + count
        set_freq()

def mouse_button_press(event):

    if event.num == 1:
        if settings[0] == 'Split':
            comport.write([0xfe, 0xfe, RIG, PC, 0x1c, 0x02, 0x01, 0xfd])
            settings[1] = 'XFC ON'
            print('press  ', event.num, 'XFC ON') 

        else:
             settings[1] = ''

    var1.set(settings[0])
    var2.set(settings[1])

def mouse_button_release(event):

    if event.num== 1 and settings[0] == 'Split':
        comport.write([0xfe, 0xfe, RIG, PC, 0x1c, 0x02, 0x00, 0xfd])
        settings[1] = 'XFC OFF'
        print('release', event.num, settings[1]) 
        var2.set(settings[1])

def exit_proc():
    if txtmsg.askokcancel('','終了しますか?'):
        comport.close()
        time.sleep(0.1)
        root.destroy()

root = tk.Tk()
root.title('ICOM Split Controller')
root.geometry("300x235")
root.protocol('WM_DELETE_WINDOW', exit_proc)

var0 = tk.StringVar() # wpm
var1 = tk.StringVar() # Simplex/Split
var2 = tk.StringVar() # XFC ON/OFF

var1.set(settings[0])

tk.Label(root, textvariable = var1, font=('', 20), width =10).pack()
tk.Label(root, textvariable = var2, font=('', 20), width =10).pack()

tk.Label(root, textvariable = var0 , fg='white', bg='black', font=('Times', 20)).pack(fill = 'both')

tk.Label(root, text = b' F1 ' + msg[0], font=('', 20), fg='white', bg='blue', anchor="w").pack(side = 'top', fill = 'both') # message 1
tk.Label(root, text = b' F2 ' + msg[1], font=('', 20), fg='white', bg='blue', anchor="w").pack(side = 'top', fill = 'both') # message 2
tk.Label(root, text = b' F3 ' + msg[2], font=('', 20), fg='white', bg='blue', anchor="w").pack(side = 'top', fill = 'both') # message 3
tk.Label(root, text = b' F4 ' + msg[3], font=('', 20), fg='white', bg='blue', anchor="w").pack(side = 'top', fill = 'both') # message 4

root.bind('<Any-KeyPress>', kbd)
root.bind("<MouseWheel>", mouse_wheel)
root.bind("<ButtonPress>", mouse_button_press)
root.bind("<ButtonRelease>", mouse_button_release)

set_wpm(0) # CW Speed初期化
time.sleep(0.01)

#comport.write([0xfe, 0xfe, RIG, PC, 0x16, 0x47, 0x01, 0xfd]) # CWセミブレークインに設定

set_simplex()

root.mainloop()
